define([
    "dojo/parser",
    "dojo/_base/window",
    "dojo/on",
    "dojo/_base/lang",
    "dojo/json",
    "dojo/request",
    "dojo/dom",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/_base/declare",
    "dijit/_WidgetBase",
    "dijit/_OnDijitClickMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/Form",
    "dijit/form/ValidationTextBox",
    "dijit/form/TextBox",
    "dijit/form/Button",
    "dijit/Dialog",

    "app/VSnpsDialog",

    "dijit/popup",
    "dijit/layout/LinkPane",
    "app/SnpsDialog",
    "dgrid/Grid",
    "dgrid/extensions/DijitRegistry",
    "dojo/text!./templates/ValidationSamplesPane.html",
    "dojo/_base/array",
    "dgrid/OnDemandGrid",
    "dgrid/editor",
    "dijit/InlineEditBox",
    "dgrid/Selection",
    "dgrid/CellSelection",
    "dgrid/Keyboard",
    "dojo/domReady!"
], function(parser, window, on, lang, json, request, dom, domStyle, domConstruct, declare, _WidgetBase, _OnDijitClickMixin, _TemplatedMixin, _WidgetsInTemplateMixin, Form, ValidationTextBox, TextBox, Button, Dialog, VSnpsDialog, popup, LinkPane, SnpsDialog, Grid, DijitRegistry, template, arrayUtil, OnDemandGrid, editor, InlineEditBox, Selection, CellSelection, Keyboard, domReady) {
 
     return declare([_WidgetBase, _OnDijitClickMixin, _TemplatedMixin, _WidgetsInTemplateMixin], {
        templateString: template,
        widgetName: "ValidationSamplesPane",

        refreshGrid: function(response)
        { 
          if(response.length == 0)
          {
            window.global.app.widgets.LogoutPane.onButtonClick();
            return;
          }
          this.vsamples_data = [];
          this.vsamples_parsed = json.parse(response);
          console.log(this.vsamples_parsed);
          for(var i=0; i<this.vsamples_parsed.length; i++) 
          {   
            var e = this.vsamples_parsed[i];
            this.vsamples_data.push(
              {
                c_gpid: e.gpid,
                c_version: e.currency,
                c_gprecord: e.gprecord_status,
                c_gprecord_details: e.gprecord,
                c_haplosnps: e.haplosnps_status,
                c_haplosnps_details: e.haplosnps,
                c_msnps: e.msnps_status,
                c_msnps_details: e.msnps,
                c_mranges: e.mranges_status,
                c_mranges_details: e.mranges,
                c_admixture: e.admixture_status,
                c_admixture_details: e.admixture,
                c_closest: e.closest_status,
                c_closest_details: e.closest
              }
            );
            if(e.currency == "backup")
            {
              this.vsamples_data.push(
                {
                  c_gpid: "",
                  c_version: "",
                  c_gprecord: "",
                  c_haplosnps: "",
                  c_msnps: "",
                  c_mranges: "",
                  c_admixture: "",
                  c_closest: ""
                }
              );
            }
          }
          this.vsamples_grid.refresh(this.vsamples_data);
          this.vsamples_grid.renderArray(this.vsamples_data);
          console.log(this.vsamples_grid);
          
          /* this.anchor.innerHTML = response; */
        },

        getWidgetContent: function(arg)
        {
          request("services/get" + this.widgetName + "Content.php?arg=" + arg).then(lang.hitch(this, this.showContent));
        },
        showContent: function(response)
        {
          this.anchor.innerHTML = response;
        },

        paneExposed: function()
        {
          console.log(this.widgetName + " exposed", this);
          request("services/get" + this.widgetName + "Content.php?arg=" + "arg").then(lang.hitch(this, this.refreshGrid));
        },

        createGrid: function()
        {
          var columns =
          [
            { field: "c_gpid", label: "GPID", id: "col1"},
            { field: "c_version", label: "Version", id: "col1a"},
            { field: "c_gprecord", label: "gprecord", id: "col2"},
            { field: "c_haplosnps", label: "haplosnps", id: "col3"},
            { field: "c_msnps", label: "msnps", id: "col4"},
            { field: "c_mranges", label: "mranges", id: "col5"},
            { field: "c_admixture", label: "admixture", id: "col6"},
            { field: "c_closest", label: "closest", id: "col7"}
          ];
          var CellSelectionGrid = declare([OnDemandGrid, CellSelection, Keyboard]);
          this.vsamples_grid = new CellSelectionGrid({columns: columns, selectionMode: "single", width: "600px"}, this.vsamples);           
          domConstruct.place("<div data-dojo-attach-point='vsamples'></div>", this.anchor, "after");
          on(this.vsamples_grid, "dgrid-select,dgrid-deselect", lang.hitch(this, this.cellSelection));
        },

        cellSelection: function(evt)
        {
          var evtType = evt.type;
          var cells = evt.cells[0];
          console.log("evt:", evt);
          console.log("length(evtType):", evtType.length);
          var column = cells.column.field;
          var rowdata = cells.row.data;
          if(evtType == "dgrid-select")
          {
            console.log("select", column, rowdata);
            if(rowdata.c_gpid != "")
            {
              if(column != "c_gpid" && column != "c_version" && column != "c_haplosnps")
              {
                this.showDetailsDialog(rowdata.c_gpid, column);
              }
              else
              {
                if(column == "c_haplosnps") this.showSnpsDialog(rowdata.c_gpid, column);
              }
            }
          }
          if(evtType == "dgrid-deselect")
          {
            console.log("deselect");
          }
        },

        showSnpsDialog: function(gpid, column)
        {
/*
          this.snpsDialog = new SnpsDialog({gpid: gpid, column: "haplosnps", data: this.vsamples_data});
          this.snpsDialog.show();
*/
          console.log("GPID - ", gpid);
          this.snpsDialog = new VSnpsDialog({gpid: gpid, column: "haplosnps", data: this.vsamples_data});
          this.snpsDialog.show();
        },

        showDetailsDialog: function(gpid, column)
        {
          var myDialog = new Dialog({
            title: "Details for " + gpid + " " + column,
            style: "width: 800px"
          });
          var dialog_data = "";
          var current = "";
          var backup = ""
          for(var i=0; i<this.vsamples_data.length; i++)
          {
            if(this.vsamples_data[i].c_gpid == gpid)
            {
              if(this.vsamples_data[i].c_version == "current") current = this.vsamples_data[i][column + "_details"];
              if(this.vsamples_data[i].c_version == "backup") backup = this.vsamples_data[i][column + "_details"];
            }
          }

          dialog_data = "<pre><table border=1><tr><td>    Current</td><td>    Backup</td></tr><tr><td>" + current + "</td><td>" + backup + "</td></tr></table></pre>";
       
          myDialog.set("content", dialog_data);
          myDialog.show();
        },

        postCreate: function()
        {
          console.log(this.widgetName + " postCreate:", this);
        },
        startup: function()
        {
          window.global.app.widgets[this.widgetName] = this;
          this.createGrid();
        }
       
    });
});
